# import os
# import tempfile
# from langchain.text_splitter import RecursiveCharacterTextSplitter
# from langchain_community.document_loaders.sitemap import SitemapLoader
# from langchain_community.vectorstores import SKLearnVectorStore
# from langchain_openai import OpenAIEmbeddings

# RAG_PROMPT = """You are an assistant for question-answering tasks.
# Use the following pieces of retrieved context to answer the latest question in the conversation.
# If you don't know the answer, just say that you don't know.
# The pre-existing conversation may provide important context to the question.
# Use three sentences maximum and keep the answer concise.
# Important: if you know / have the URL of the source document, add it to your answer

# Conversation: {conversation}
# Context: {context}
# Question: {question}
# Answer:"""


# def get_vector_db_retriever():
#     persist_path = os.path.join(tempfile.gettempdir(), "union.parquet")
#     embd = OpenAIEmbeddings()

#     # If vector store exists, then load it
#     if os.path.exists(persist_path):
#         vectorstore = SKLearnVectorStore(embedding=embd, persist_path=persist_path, serializer="parquet")
#         return vectorstore.as_retriever(lambda_mult=0)

#     # Otherwise, index LangSmith documents and create new vector store
#     ls_docs_sitemap_loader = SitemapLoader(web_path="https://docs.smith.langchain.com/sitemap.xml")
#     ls_docs = ls_docs_sitemap_loader.load()

#     text_splitter = RecursiveCharacterTextSplitter.from_tiktoken_encoder(chunk_size=500, chunk_overlap=0)
#     doc_splits = text_splitter.split_documents(ls_docs)

#     vectorstore = SKLearnVectorStore.from_documents(
#         documents=doc_splits, embedding=embd, persist_path=persist_path, serializer="parquet"
#     )
#     vectorstore.persist()
#     return vectorstore.as_retriever(lambda_mult=0)
